package com.allinpay.yst2.demo.tx;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.net.ssl.HttpsURLConnection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.allinpay.yst2.demo.config.DemoConfig;
import com.allinpay.yst2.demo.util.DemoSM2Utils;
import com.allinpay.yst2.demo.util.TxTrustManager;
import com.allinpay.yst2.demo.vo.BizParameter;
import com.allinpay.yst2.demo.vo.TxRequest;
import com.allinpay.yst2.demo.vo.TxResponse;

/**
 * 测试客户端
 *
 * @author gejunqing
 * @version 1.0
 * @date 2024/1/11
 */
public class TxClient
{
    private final Logger logger = LoggerFactory.getLogger(TxClient.class);
    private static final String YYYY_MM_DD = "yyyyMMdd";
    private static final String HH_MM_SS = "HHmmss";
    private final DemoConfig config;
    private final PrivateKey privateKey;
    private final PublicKey tlPublicKey;

    public TxClient(DemoConfig config) throws Exception
    {
        this.config = config;
        this.privateKey = DemoSM2Utils.privKeySM2FromBase64Str(config.getPrivateKeyStr());
        this.tlPublicKey = DemoSM2Utils.pubKeySM2FromBase64Str(config.getAllinpayPublicKeyStr());
    }

    public TxResponse sendRequest(String transCode, BizParameter param) throws Exception
    {
        TxRequest request = assembleRequest(transCode, param);
        logger.info("request:{}", request);
        String respStr = post(request.toString());
        logger.info("response:{}", respStr);
        verify(respStr);
        logger.info("验签成功");
        return JSON.parseObject(respStr, TxResponse.class);
    }

    public TxResponse sendRequest(String transCode, BizParameter param, String url) throws Exception
    {
        TxRequest request = assembleRequest(transCode, param);
        logger.info("request:{}", request);
        String respStr = post(request.toString(), url);
        logger.info("response:{}", respStr);
        verify(respStr);
        logger.info("验签成功");
        return JSON.parseObject(respStr, TxResponse.class);
    }

    private void verify(String respStr) throws Exception
    {
        JSONObject map = JSON.parseObject(respStr);
        String sign = (String) map.remove("sign");
        String signType = (String) map.remove("signType");
        String srcSignMsg = DemoSM2Utils.jsonMapToStr(map);
        if (!DemoSM2Utils.verify(this.tlPublicKey, srcSignMsg, sign))
        {
            throw new Exception("响应报文验签失败");
        }
    }

    private TxRequest assembleRequest(String transCode, BizParameter param) throws Exception
    {
        TxRequest request = new TxRequest();
        request.setAppId(this.config.getAppId());
        request.setSpAppId(this.config.getSpAppId());
        request.setTransCode(transCode);
        request.setFormat(this.config.getFormat());
        request.setCharset(this.config.getCharset());
        request.setTransDate(new SimpleDateFormat(YYYY_MM_DD).format(new Date()));
        request.setTransTime(new SimpleDateFormat(HH_MM_SS).format(new Date()));
        request.setVersion(this.config.getVersion());
        request.setBizData(param.toString());
        String signedValue = DemoSM2Utils.jsonMapToStr((JSONObject) JSON.toJSON(request));
        logger.info("待签名源串:{}", signedValue);
        String sign = DemoSM2Utils.sign(this.privateKey, signedValue);
        request.setSignType(this.config.getSignType());
        request.setSign(sign);
        return request;
    }

    /**
     * 发送请求
     * 
     * @param param
     * @return
     * @throws IOException
     */
    private String post(String param) throws IOException
    {
        return post(param, getReqUrl());
    }

    /**
     * 发送请求
     *
     * @param param
     * @return
     * @throws IOException
     */
    private String post(String param, String url) throws IOException
    {
        StringBuilder result = new StringBuilder();
        BufferedWriter writer = null;
        BufferedReader reader = null;

        try
        {
            URL httpUrl = new URL(url == null ? getReqUrl() : url);
            HttpURLConnection connection = (HttpURLConnection) httpUrl.openConnection();
            if (connection instanceof HttpsURLConnection)
            {
                ((HttpsURLConnection) connection).setSSLSocketFactory(TxTrustManager.instance().getSSLSocketFactory());
            }
            connection.setRequestProperty("Connection", "keep-alive");
            connection.setRequestProperty("Content-Type", "application/json;charset=utf-8");
            connection.setRequestMethod("POST");
            connection.setDoOutput(true);
            connection.setDoInput(true);
            writer = new BufferedWriter(new OutputStreamWriter(connection.getOutputStream(), StandardCharsets.UTF_8));
            writer.write(param);
            writer.flush();
            // 响应
            reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), StandardCharsets.UTF_8));
            while (true)
            {
                String line = reader.readLine();
                if (line == null)
                {
                    break;
                }
                result.append(line);
            }
        }
        catch (Exception e2)
        {
            this.logger.error(e2.getMessage(), e2);
        }
        finally
        {
            if (writer != null)
            {
                try
                {
                    writer.close();
                }
                catch (IOException e)
                {
                    this.logger.error(e.getMessage(), e);
                }
            }
            if (reader != null)
            {
                reader.close();
            }
        }
        return result.toString();
    }

    protected String getReqUrl()
    {
        return this.config.getUrl();
    }

    public DemoConfig getConfig()
    {
        return config;
    }
}
